"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.os = undefined;

var _dogmalangmin = require("dogmalangmin");

Object.keys(_dogmalangmin).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _dogmalangmin[key];
    }
  });
});
exports.exec = exec;
exports.execf = execf;
exports.pexec = pexec;

var _child_process = require("child_process");

var _child_process2 = _interopRequireDefault(_child_process);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * The OS object.
 */
const os = exports.os = {
  release() {
    const rls = require("fs").readFileSync("/etc/os-release", "utf-8");

    return {
      id: /ID=(.+)/.exec(rls)[1],
      version: /VERSION_ID="(.+)"/.exec(rls)[1]
    };
  }
};

/**
 * The process object.
 */
//imports
_dogmalangmin.ps.exit = process.exit;
_dogmalangmin.ps.cd = process.chdir;
_dogmalangmin.ps.on = process.on.bind(process);

/**
 * exec() function.
 */
function exec(...args) {
  var cmd,
      opts = {},
      res;
  var last = args[args.length - 1],
      props;

  //(1) get user options/properties and command
  if (args.length > 1 && (last instanceof Function || last instanceof Object)) {
    cmd = args.slice(0, -1).join(" ");
    props = last instanceof Function ? { done: last } : last;
  } else {
    cmd = args.join(" ");
    props = {};
  }

  //(2) get exec options
  opts.cwd = props.wd || props.workDir;
  opts.env = props.env;
  if (props.detach || props.done) opts.detached = true;

  const enc = props.encoding || props.enc;
  opts.encoding = enc == "buffer" ? "buffer" : "utf-8";
  opts.stdio = ["inherit", "pipe", "pipe"];

  //(3) exec
  if (opts.detached) {
    res = _child_process2.default.exec(cmd, opts, props.done || function () {});
    res.unref();
  } else if (props.async) {
    res = new Promise(function (rslv, rej) {
      _child_process2.default.exec(cmd, opts, function (err, stdout, stderr) {
        if (err) rej(err);else rslv(stdout, stderr);
      });
    });
  } else {
    res = _child_process2.default.execSync(cmd, opts);
  }

  //(4) return
  return res;
}

/**
 * execf() function.
 */
function execf(...args) {
  var cmd,
      props,
      last = args[args.length - 1];

  if (last instanceof Function || last instanceof Object) {
    cmd = (0, _dogmalangmin.fmt)(...args.slice(0, -1));
    props = last;
  } else {
    cmd = (0, _dogmalangmin.fmt)(...args);
  }

  return exec(cmd, props);
}

/**
 * pexec() function.
 */
function pexec(...cmd) {
  try {
    return [true, exec(...cmd)];
  } catch (e) {
    return [false, e];
  }
}

/**
 * pwait() function.
 */
_dogmalangmin.dogma.pwait = function pwait(fn) {
  const deasync = require("deasync");
  var ok, res;

  try {
    res = deasync(fn)();
    ok = true;
  } catch (e) {
    ok = false;
    res = e;
  }

  return [ok, res];
};